@@ -24,9 +24,7 @@ dnf_install() {
 install_ramalama() {
   # link podman-remote to podman for use by RamaLama
   ln -sf /usr/bin/podman-remote /usr/bin/podman
-  if [ -e "/run/ramalama" ]; then
-    python3 -m pip install /run/ramalama --prefix=/usr
-  fi
+  python3 -m pip install .
 }
 
 main() {
