@@ -1,22 +1,11 @@
 #!/bin/bash
 
-python_version() {
-  local pyversion
-  pyversion=$(python3 --version)
-  # $2 is empty when no Python is installed, so just install python3
-  if [ -n "$pyversion" ]; then
-    local pystr="$pyversion
-Python 3.10"
-    if [ "$pystr" == "$(sort --version-sort <<<"$pystr")" ]; then
-      echo "python3.11"
-      return
-    fi
-  fi
-  echo "python3"
-}
+DEFAULT_LLAMA_CPP_COMMIT="b52edd25586fabb70f0c21b274473b307cf14499"
+DEFAULT_WHISPER_COMMIT="c62adfbd1ecdaea9e295c72d672992514a2d887c"
 
 dnf_install_intel_gpu() {
   local intel_rpms=("intel-oneapi-mkl-sycl-devel" "intel-oneapi-dnnl-devel"
+    "intel-oneapi-mkl-devel" "intel-oneapi-mkl-sycl-distributed-dft-devel"
     "intel-oneapi-compiler-dpcpp-cpp" "intel-level-zero"
     "oneapi-level-zero" "oneapi-level-zero-devel" "intel-compute-runtime")
   dnf install -y "${intel_rpms[@]}"
@@ -66,17 +55,16 @@ dnf_install_rocm() {
   rm_non_ubi_repos
 }
 
-dnf_install_s390() {
-  # I think this was for s390, maybe ppc also
+dnf_install_s390_ppc64le() {
   dnf install -y "openblas-devel"
 }
 
 dnf_install_mesa() {
   if [ "${ID}" = "fedora" ]; then
     dnf copr enable -y slp/mesa-libkrun-vulkan
-    dnf install -y mesa-vulkan-drivers-25.0.7-101.fc42 virglrenderer \
+    dnf install -y mesa-vulkan-drivers-25.2.3-101.fc43 virglrenderer \
       "${vulkan_rpms[@]}"
-    dnf versionlock add mesa-vulkan-drivers-25.0.7-101.fc42
+    dnf versionlock add mesa-vulkan-drivers-25.2.3-101.fc43
   elif [ "${ID}" = "openEuler" ]; then
     dnf install -y mesa-vulkan-drivers virglrenderer "${vulkan_rpms[@]}"
   else # virglrenderer not available on RHEL or EPEL
@@ -107,9 +95,8 @@ dnf_install_ffmpeg() {
 
 dnf_install() {
   local rpm_exclude_list="selinux-policy,container-selinux"
-  local rpm_list=("${PYTHON}" "${PYTHON}-pip"
-    "python3-argcomplete" "python3-dnf-plugin-versionlock"
-    "${PYTHON}-devel" "gcc-c++" "cmake" "vim" "procps-ng" "git-core"
+  local rpm_list=("python3-dnf-plugin-versionlock"
+    "gcc-c++" "cmake" "vim" "procps-ng" "git-core"
     "dnf-plugins-core" "libcurl-devel" "gawk")
   local vulkan_rpms=("vulkan-headers" "vulkan-loader-devel" "vulkan-tools"
     "spirv-tools" "glslc" "glslang")
@@ -119,14 +106,11 @@ dnf_install() {
   else
     dnf install -y "${rpm_list[@]}" --exclude "${rpm_exclude_list}"
   fi
-  if [[ "${PYTHON}" == "python3.11" ]]; then
-    ln -sf /usr/bin/python3.11 /usr/bin/python3
-  fi
   if [ "$containerfile" = "ramalama" ]; then
     if [ "$uname_m" = "x86_64" ] || [ "$uname_m" = "aarch64" ]; then
       dnf_install_mesa # on x86_64 and aarch64 we use vulkan via mesa
     else
-      dnf_install_s390
+      dnf_install_s390_ppc64le
     fi
   elif [[ "$containerfile" = rocm* ]]; then
     dnf_install_rocm
@@ -182,14 +166,19 @@ setup_build_env() {
 }
 
 cmake_steps() {
-  local cmake_flags=("$@")
-  cmake -B build "${cmake_flags[@]}" 2>&1 | cmake_check_warnings
-  local build_config=Release
-  if [[ "${RAMALAMA_IMAGE_BUILD_DEBUG_MODE:-}" == y ]]; then
-      build_config=Debug
-  fi
-  cmake --build build --config "$build_config" -j"$(nproc)" 2>&1 | cmake_check_warnings
-  cmake --install build 2>&1 | cmake_check_warnings
+  (
+    # This makes llama.cpp build a generic binary, similar to an rpm build
+    SOURCE_DATE_EPOCH=$(git log -1 --pretty=%ct)
+    export SOURCE_DATE_EPOCH
+    local cmake_flags=("$@")
+    cmake -B build "${cmake_flags[@]}" 2>&1 | cmake_check_warnings
+    local build_config=Release
+    if [[ "${RAMALAMA_IMAGE_BUILD_DEBUG_MODE:-}" == y ]]; then
+        build_config=Debug
+    fi
+    cmake --build build --config "$build_config" -j"$(nproc)" 2>&1 | cmake_check_warnings
+    cmake --install build 2>&1 | cmake_check_warnings
+  )
 }
 
 set_install_prefix() {
@@ -201,7 +190,7 @@ set_install_prefix() {
 }
 
 configure_common_flags() {
-  common_flags=("-DGGML_NATIVE=OFF")
+  common_flags=()
   if [[ "${RAMALAMA_IMAGE_BUILD_DEBUG_MODE:-}" == y ]]; then
       common_flags+=("-DGGML_CMAKE_BUILD_TYPE=Debug")
   else
@@ -235,7 +224,6 @@ configure_common_flags() {
 }
 
 clone_and_build_whisper_cpp() {
-  local DEFAULT_WHISPER_COMMIT="7745fcf32846006128f16de429cfe1677c963b30"
   local whisper_cpp_commit="${WHISPER_CPP_PULL_REF:-$DEFAULT_WHISPER_COMMIT}"
   local whisper_flags=("${common_flags[@]}")
   whisper_flags+=("-DBUILD_SHARED_LIBS=OFF")
@@ -254,7 +242,6 @@ clone_and_build_whisper_cpp() {
 }
 
 clone_and_build_llama_cpp() {
-  local DEFAULT_LLAMA_CPP_COMMIT="b0d52998b962bd2681c34bf52af993af79f178b8"
   local llama_cpp_commit="${LLAMA_CPP_PULL_REF:-$DEFAULT_LLAMA_CPP_COMMIT}"
   local install_prefix
   install_prefix=$(set_install_prefix)
@@ -267,25 +254,6 @@ clone_and_build_llama_cpp() {
   fi
 }
 
-install_ramalama() {
-  if [ -e "pyproject.toml" ]; then
-    $PYTHON -m pip install . --prefix="$1"
-  fi
-}
-
-install_entrypoints() {
-  if [ -e "container-images" ]; then
-    install -d "$install_prefix"/bin
-    install -m 755 \
-      container-images/scripts/llama-server.sh \
-      container-images/scripts/whisper-server.sh \
-      container-images/scripts/build_rag.sh \
-      container-images/scripts/doc2rag \
-      container-images/scripts/rag_framework \
-      "$install_prefix"/bin
-  fi
-}
-
 cleanup() {
   available dnf && dnf_remove
   rm -rf /var/cache/*dnf* /opt/rocm-*/lib/*/library/*gfx9*
@@ -298,9 +266,11 @@ add_common_flags() {
   ramalama)
     if [ "$uname_m" = "x86_64" ] || [ "$uname_m" = "aarch64" ]; then
       common_flags+=("-DGGML_VULKAN=ON")
-    elif [ "$uname_m" = "s390x" ]; then
-      common_flags+=("-DGGML_VXE=ON" "-DGGML_BLAS=ON")
-      common_flags+=("-DGGML_BLAS_VENDOR=OpenBLAS")
+    elif [ "$uname_m" = "s390x" ] || [ "$uname_m" = "ppc64le" ]; then
+      common_flags+=("-DGGML_BLAS=ON" "-DGGML_BLAS_VENDOR=OpenBLAS")
+    fi
+    if [ "$uname_m" = "s390x" ]; then
+      common_flags+=("-DARCH_FLAGS=-march=z15")
     fi
     ;;
   esac
@@ -311,11 +281,9 @@ main() {
   source /etc/os-release
 
   set -ex -o pipefail
-  export PYTHON
-  PYTHON=$(python_version)
 
   # shellcheck disable=SC1091
-  source container-images/scripts/lib.sh
+  source "$(dirname "$0")/lib.sh"
 
   local containerfile=${1-""}
   local install_prefix
@@ -326,11 +294,7 @@ main() {
   configure_common_flags
   common_flags+=("-DGGML_CCACHE=OFF" "-DCMAKE_INSTALL_PREFIX=${install_prefix}")
   available dnf && dnf_install
-  if [ -n "$containerfile" ]; then
-    install_ramalama "${install_prefix}"
-  fi
 
-  install_entrypoints
   setup_build_env
   if [ "$uname_m" != "s390x" ]; then
     clone_and_build_whisper_cpp
