@@ -1,4 +1,5 @@
 import argparse
+import copy
 import errno
 import json
 import os
@@ -7,10 +8,9 @@ import subprocess
 import sys
 import urllib.error
 from datetime import datetime, timezone
-from pathlib import Path
+from textwrap import dedent
 from typing import get_args
-
-from ramalama.config import COLOR_OPTIONS, SUPPORTED_ENGINES, SUPPORTED_RUNTIMES
+from urllib.parse import urlparse
 
 # if autocomplete doesn't exist, just do nothing, don't break
 try:
@@ -24,13 +24,26 @@ except Exception:
 import ramalama.chat as chat
 from ramalama import engine
 from ramalama.chat import default_prefix
+from ramalama.cli_arg_normalization import normalize_pull_arg
+from ramalama.command.factory import assemble_command
 from ramalama.common import accel_image, get_accel, perror
-from ramalama.config import CONFIG, coerce_to_bool, load_file_config
+from ramalama.config import (
+    COLOR_OPTIONS,
+    CONFIG,
+    GGUF_QUANTIZATION_MODES,
+    SUPPORTED_ENGINES,
+    SUPPORTED_RUNTIMES,
+    coerce_to_bool,
+    get_inference_schema_files,
+    get_inference_spec_files,
+    load_file_config,
+)
 from ramalama.endian import EndianMismatchError
+from ramalama.log_levels import LogLevel
 from ramalama.logger import configure_logger, logger
 from ramalama.model_inspect.error import ParseError
 from ramalama.model_store.global_store import GlobalModelStore
-from ramalama.rag import Rag, rag_image
+from ramalama.rag import INPUT_DIR, Rag, RagTransport, rag_image
 from ramalama.shortnames import Shortnames
 from ramalama.stack import Stack
 from ramalama.transports.base import (
@@ -38,6 +51,7 @@ from ramalama.transports.base import (
     NoGGUFModelFileFound,
     NoRefFileFound,
     SafetensorModelNotSupported,
+    compute_serving_port,
     trim_model_name,
 )
 from ramalama.transports.transport_factory import New, TransportFactory
@@ -45,7 +59,6 @@ from ramalama.version import print_version, version
 
 shortnames = Shortnames()
 
-
 GENERATE_OPTIONS = ["quadlet", "kube", "quadlet/kube", "compose"]
 
 
@@ -79,7 +92,7 @@ def parse_port_option(option: str) -> str:
     port = int(option)
     if port <= 0 or port >= 65535:
         raise ValueError(f"Invalid port '{port}'")
-    return str(port)
+    return option
 
 
 class OverrideDefaultAction(argparse.Action):
@@ -120,8 +133,6 @@ def available_metadata(prefix, parsed_args, **kwargs):
         # shotnames resolution has not been applied for auto-completion
         # Therefore it needs to be done explicitly here in order to support it
         resolved_model = shortnames.resolve(parsed_args.MODEL)
-        if not resolved_model:
-            resolved_model = parsed_args.MODEL
 
         metadata = New(resolved_model, parsed_args).inspect_metadata()
         return [field for field in metadata.keys() if field.startswith(parsed_args.get)]
@@ -161,6 +172,18 @@ def init_cli():
     return parser, args
 
 
+def parse_args_from_cmd(cmd: list[str]) -> argparse.Namespace:
+    """Parse arguments based on a command string"""
+    # Need to know if we're running with --dryrun or --generate before adding the subcommands,
+    # otherwise calls to accel_image() when setting option defaults will cause unnecessary image pulls.
+    if any(arg in ("--dryrun", "--dry-run", "--generate") or arg.startswith("--generate=") for arg in sys.argv[1:]):
+        CONFIG.dryrun = True
+    parser = get_parser()
+    args = parser.parse_args(cmd)
+    post_parse_setup(args)
+    return args
+
+
 def get_description():
     """Return the description of the RamaLama tool."""
     return """\
@@ -288,12 +311,38 @@ def parse_arguments(parser):
 
 def post_parse_setup(args):
     """Perform additional setup after parsing arguments."""
+
+    def map_https_to_transport(input: str) -> str | None:
+        if input.startswith("https://") or input.startswith("http://"):
+            url = urlparse(input)
+            # detect if the whole repo is defined or a specific file
+            if url.path.count("/") != 2:
+                return input
+            if url.hostname in ["hf.co", "huggingface.co"]:
+                return f"hf:/{url.path}"
+            if url.hostname in ["ollama.com"]:
+                return f"ollama:/{url.path}"
+        return input
+
+    # Resolve the model input
+    # First, map https:// inputs to ollama or huggingface based on url domain
+    # Then resolve the input based on the available shortname list
     if getattr(args, "MODEL", None):
-        if args.subcommand != "rm":
-            resolved_model = shortnames.resolve(args.MODEL)
-            if resolved_model:
-                args.UNRESOLVED_MODEL = args.MODEL
-                args.MODEL = resolved_model
+        if isinstance(args.MODEL, str):
+            args.INITIAL_MODEL = args.MODEL
+            args.MODEL = map_https_to_transport(args.MODEL)
+
+            args.UNRESOLVED_MODEL = args.MODEL
+            args.MODEL = shortnames.resolve(args.MODEL)
+
+        if isinstance(args.MODEL, list):
+            args.INITIAL_MODEL = [m for m in args.MODEL]
+            for i in range(len(args.MODEL)):
+                args.MODEL[i] = map_https_to_transport(args.MODEL[i])
+
+            args.UNRESOLVED_MODEL = [m for m in args.MODEL]
+            for i in range(len(args.MODEL)):
+                args.MODEL[i] = shortnames.resolve(args.MODEL[i])
 
         # TODO: This is a hack. Once we have more typing in place these special cases
         # _should_ be removed.
@@ -318,7 +367,11 @@ def post_parse_setup(args):
             logger.info("MLX runtime automatically uses --nocontainer mode")
         args.container = False
 
-    configure_logger("DEBUG" if args.debug else "WARNING")
+    if hasattr(args, 'pull'):
+        args.pull = normalize_pull_arg(args.pull, getattr(args, 'engine', None))
+
+    log_level = LogLevel.DEBUG if args.debug else (CONFIG.log_level or LogLevel.WARNING)
+    configure_logger(log_level)
 
 
 def login_parser(subparsers):
@@ -407,27 +460,10 @@ def human_duration(d):
     return "1 year" if d < 63072000 else f"{d // 31536000} years"
 
 
-def list_files_by_modification(args):
-    paths = Path().rglob("*")
-    models = []
-    for path in paths:
-        if str(path).startswith("file/"):
-            if not os.path.exists(str(path)):
-                path = str(path).replace("file/", "file:///")
-                perror(f"{path} does not exist")
-                continue
-        if os.path.exists(path):
-            models.append(path)
-        else:
-            perror(f"Broken symlink found in: {args.store}/models/{path} \nAttempting removal")
-            New(str(path).replace("/", "://", 1), args).remove(args)
-
-    return sorted(models, key=lambda p: os.path.getmtime(p), reverse=True)
-
-
 def bench_cli(args):
     model = New(args.MODEL, args)
-    model.bench(args)
+    model.ensure_model_exists(args)
+    model.bench(args, assemble_command(args))
 
 
 def add_network_argument(parser, dflt="none"):
@@ -502,18 +538,6 @@ def human_readable_size(size):
     return f"{size} PB"
 
 
-def get_size(path):
-    if os.path.isdir(path):
-        size = 0
-        for dirpath, _, filenames in os.walk(path, followlinks=True):
-            for file in filenames:
-                filepath = os.path.join(dirpath, file)
-                if not os.path.islink(filepath):
-                    size += os.path.getsize(filepath)
-        return size
-    return os.path.getsize(path)
-
-
 def _list_models_from_store(args):
     models = GlobalModelStore(args.store).list_models(engine=args.engine, show_container=args.container)
 
@@ -549,25 +573,30 @@ def _list_models(args):
 
 def info_cli(args):
     info = {
+        "Accelerator": get_accel(),
         "Config": load_file_config(),
         "Engine": {
             "Name": args.engine,
         },
         "Image": accel_image(CONFIG),
-        "Runtime": args.runtime,
+        "Inference": {
+            "Default": args.runtime,
+            "Engines": {spec: str(path) for spec, path in get_inference_spec_files().items()},
+            "Schema": {schema: str(path) for schema, path in get_inference_schema_files().items()},
+        },
+        "RagImage": rag_image(CONFIG),
         "Selinux": CONFIG.selinux,
+        "Shortnames": {
+            "Files": shortnames.paths,
+            "Names": shortnames.shortnames,
+        },
         "Store": args.store,
         "UseContainer": args.container,
         "Version": version(),
     }
-    info["Shortnames"] = {
-        "Files": shortnames.paths,
-        "Names": shortnames.shortnames,
-    }
     if args.engine and len(args.engine) > 0:
         info["Engine"]["Info"] = engine.info(args)
 
-    info["Accelerator"] = get_accel()
     print(json.dumps(info, sort_keys=True, indent=4))
 
 
@@ -654,23 +683,35 @@ def convert_parser(subparsers):
     )
     parser.add_argument(
         "--gguf",
-        choices=[
-            "Q2_K",
-            "Q3_K_S",
-            "Q3_K_M",
-            "Q3_K_L",
-            "Q4_0",
-            "Q4_K_S",
-            "Q4_K_M",
-            "Q5_0",
-            "Q5_K_S",
-            "Q5_K_M",
-            "Q6_K",
-            "Q8_0",
-        ],
-        help="GGUF quantization format",
+        choices=get_args(GGUF_QUANTIZATION_MODES),
+        nargs="?",
+        const=CONFIG.gguf_quantization_mode,  # Used if --gguf is provided without value
+        default=None,  # Used if --gguf is not provided
+        help=f"GGUF quantization format. If specified without value, {CONFIG.gguf_quantization_mode} is used.",
     )
     add_network_argument(parser)
+    parser.add_argument(
+        "--rag-image",
+        default=rag_image(CONFIG),
+        help="Image to use for conversion to GGUF",
+        action=OverrideDefaultAction,
+        completer=local_images,
+    )
+    parser.add_argument(
+        "--image",
+        default=accel_image(CONFIG),
+        help="Image to use for quantization",
+        action=OverrideDefaultAction,
+        completer=local_images,
+    )
+    parser.add_argument(
+        "--pull",
+        dest="pull",
+        type=str,
+        default=CONFIG.pull,
+        choices=["always", "missing", "never", "newer"],
+        help="pull image policy",
+    )
     parser.add_argument(
         "--type",
         default="raw",
@@ -692,13 +733,10 @@ def convert_cli(args):
 
     target = args.TARGET
     tgt = shortnames.resolve(target)
-    if not tgt:
-        tgt = target
 
     model = TransportFactory(tgt, args).create_oci()
 
     source_model = _get_source_model(args)
-    args.carimage = rag_image(accel_image(CONFIG))
     model.convert(source_model, args)
 
 
@@ -738,8 +776,6 @@ Model "raw" contains the model and a link file model.file to it stored at /.""",
 
 def _get_source_model(args):
     src = shortnames.resolve(args.SOURCE)
-    if not src:
-        src = args.SOURCE
     smodel = New(src, args)
     if smodel.type == "OCI":
         raise ValueError(f"converting from an OCI based image {src} is not supported")
@@ -753,8 +789,6 @@ def push_cli(args):
     target = args.SOURCE
     if args.TARGET:
         target = shortnames.resolve(args.TARGET)
-        if not target:
-            target = args.TARGET
     target_model = New(target, args)
 
     try:
@@ -876,6 +910,15 @@ If GPU device on host is accessible to via group access, this option leaks the u
         help="name of container in which the Model will be run",
         completer=suppressCompleter,
     )
+    if command in ["run", "perplexity", "serve"]:
+        parser.add_argument(
+            "--max-tokens",
+            dest="max_tokens",
+            type=int,
+            default=CONFIG.max_tokens,
+            help="maximum number of tokens to generate (0 = unlimited)",
+            completer=suppressCompleter,
+        )
     add_network_argument(parser, dflt=None)
     parser.add_argument(
         "--ngl",
@@ -896,12 +939,13 @@ If GPU device on host is accessible to via group access, this option leaks the u
         help="override the default OCI runtime used to launch the container",
         completer=suppressCompleter,
     )
-    if command == "serve":
+    if command in ["run", "serve"]:
         parser.add_argument(
             "-p",
             "--port",
             type=parse_port_option,
             default=CONFIG.port,
+            action=OverrideDefaultAction,
             help="port for AI Model server to listen on",
             completer=suppressCompleter,
         )
@@ -916,10 +960,17 @@ If GPU device on host is accessible to via group access, this option leaks the u
         choices=["always", "missing", "never", "newer"],
         help='pull image policy',
     )
-    if command in ["serve"]:
+    if command in ["run", "serve"]:
         parser.add_argument(
             "--rag", help="RAG vector database or OCI Image to be served with the model", completer=local_models
         )
+        parser.add_argument(
+            "--rag-image",
+            default=rag_image(CONFIG),
+            help="OCI container image to run with the specified RAG data",
+            action=OverrideDefaultAction,
+            completer=local_images,
+        )
     if command in ["perplexity", "run", "serve"]:
         parser.add_argument(
             "--runtime-args",
@@ -994,8 +1045,14 @@ def chat_run_options(parser):
         help='possible values are "never", "always" and "auto".',
     )
     parser.add_argument("--prefix", type=str, help="prefix for the user prompt", default=default_prefix())
-    parser.add_argument("--rag", type=str, help="a file or directory to use as context for the chat")
     parser.add_argument("--mcp", nargs="*", help="MCP servers to use for the chat")
+    parser.add_argument(
+        "--summarize-after",
+        type=int,
+        default=CONFIG.summarize_after,
+        metavar="N",
+        help="automatically summarize conversation history after N messages to prevent context growth (0=disabled)",
+    )
 
 
 def chat_parser(subparsers):
@@ -1003,8 +1060,9 @@ def chat_parser(subparsers):
     parser.add_argument(
         "--api-key",
         type=str,
-        default=os.getenv("API_KEY"),
-        help="OpenAI-compatible API key. Can also be set via the API_KEY environment variable.",
+        default=CONFIG.api_key,
+        help="""OpenAI-compatible API key.
+        Can also be set in ramalama.conf or via the RAMALAMA_API_KEY environment variable.""",
     )
     chat_run_options(parser)
     parser.add_argument(
@@ -1015,12 +1073,34 @@ def chat_parser(subparsers):
     )
     parser.add_argument("--url", type=str, default="http://127.0.0.1:8080/v1", help="the url to send requests to")
     parser.add_argument("--model", "-m", type=str, completer=local_models, help="model for inferencing")
+    parser.add_argument("--rag", type=str, help="a file or directory to use as context for the chat")
     parser.add_argument(
         "ARGS", nargs="*", help="overrides the default prompt, and the output is returned without entering the chatbot"
     )
     parser.set_defaults(func=chat.chat)
 
 
+def _rag_args(args):
+    args.noout = not args.debug
+    rag_args = copy.copy(args)
+    rag_args.subcommand = f"{args.subcommand} --rag"
+    rag_args.MODEL = args.rag
+    rag_args.image = args.rag_image
+    if args.engine == "podman":
+        rag_args.model_host = "host.containers.internal"
+    else:
+        rag_args.model_host = f"host.{args.engine}.internal"
+    # If --name was specified, use it for the RAG proxy
+    args.name = None
+    # If --port was specified, use it for the RAG proxy, and
+    # select a random port for the model
+    args.port = None
+    rag_args.model_port = args.port = compute_serving_port(args, exclude=[rag_args.port])
+    rag_args.model_args = args
+    rag_args.generate = ""
+    return rag_args
+
+
 def run_parser(subparsers):
     parser = subparsers.add_parser("run", help="run specified AI Model as a chatbot")
     runtime_options(parser, "run")
@@ -1038,30 +1118,30 @@ def run_parser(subparsers):
 
 
 def run_cli(args):
-    if args.rag:
-        _get_rag(args)
-        # Passing default args for serve (added network bridge for internet access)
-        args.port = CONFIG.port
-        args.host = CONFIG.host
-        args.network = 'bridge'
-        args.generate = None
-
     try:
+        # detect available port and update arguments
+        args.port = compute_serving_port(args)
+
         model = New(args.MODEL, args)
         model.ensure_model_exists(args)
-        model.serve(args, quiet=True) if args.rag else model.run(args)
-
     except KeyError as e:
         logger.debug(e)
         try:
             args.quiet = True
-            oci_model = TransportFactory(args.MODEL, args, ignore_stderr=True).create_oci()
-            oci_model.ensure_model_exists(args)
-
-            oci_model.serve(args, quiet=True) if args.rag else oci_model.run(args)
+            model = TransportFactory(args.MODEL, args, ignore_stderr=True).create_oci()
+            model.ensure_model_exists(args)
         except Exception as exc:
             raise e from exc
 
+    if args.rag:
+        if not args.container:
+            raise ValueError("ramalama run --rag cannot be run with the --nocontainer option.")
+        args = _rag_args(args)
+        model = RagTransport(model, assemble_command(args.model_args), args)
+        model.ensure_model_exists(args)
+
+    model.run(args, assemble_command(args))
+
 
 def serve_parser(subparsers):
     parser = subparsers.add_parser("serve", help="serve REST API on specified AI Model")
@@ -1070,23 +1150,10 @@ def serve_parser(subparsers):
     parser.set_defaults(func=serve_cli)
 
 
-def _get_rag(args):
-    if os.path.exists(args.rag):
-        return
-    if args.pull == "never" or args.dryrun:
-        return
-    model = New(args.rag, args=args, transport="oci")
-    if not model.exists():
-        model.pull(args)
-
-
 def serve_cli(args):
     if not args.container:
         args.detach = False
 
-    if args.rag:
-        _get_rag(args)
-
     if args.api == "llama-stack":
         if not args.container:
             raise ValueError("ramalama serve --api llama-stack command cannot be run with the --nocontainer option.")
@@ -1095,16 +1162,28 @@ def serve_cli(args):
         return stack.serve()
 
     try:
+        # detect available port and update arguments
+        args.port = compute_serving_port(args)
+
         model = New(args.MODEL, args)
-        model.serve(args)
+        model.ensure_model_exists(args)
     except KeyError as e:
         try:
             args.quiet = True
             model = TransportFactory(args.MODEL, args, ignore_stderr=True).create_oci()
-            model.serve(args)
+            model.ensure_model_exists(args)
         except Exception:
             raise e
 
+    if args.rag:
+        if not args.container:
+            raise ValueError("ramalama serve --rag cannot be run with the --nocontainer option.")
+        args = _rag_args(args)
+        model = RagTransport(model, assemble_command(args.model_args), args)
+        model.ensure_model_exists(args)
+
+    model.serve(args, assemble_command(args))
+
 
 def stop_parser(subparsers):
     parser = subparsers.add_parser("stop", help="stop named container that is running AI Model")
@@ -1129,7 +1208,7 @@ def stop_container(args):
         engine.stop_container(args, i)
 
 
-def daemon_parser(subparsers):
+def daemon_parser(subparsers) -> None:
     parser: ArgumentParserWithDefaults = subparsers.add_parser("daemon", help="daemon operations")
     parser.set_defaults(func=lambda _: parser.print_help())
 
@@ -1197,7 +1276,7 @@ def daemon_start_cli(args):
         daemon_model_store_dir = "/ramalama/models"
 
         daemon_cmd += [
-            "podman",
+            args.engine,
             "run",
             "--pull",
             args.pull,
@@ -1220,7 +1299,6 @@ def daemon_start_cli(args):
         "--host",
         CONFIG.host if is_daemon_in_container else args.host,
     ]
-
     exec_cmd(daemon_cmd)
 
 
@@ -1235,6 +1313,18 @@ def version_parser(subparsers):
     parser.set_defaults(func=print_version)
 
 
+class AddPathOrUrl(argparse.Action):
+    def __call__(self, parser, namespace, values, option_string=None):
+        setattr(namespace, self.dest, [])
+        namespace.urls = []
+        for value in values:
+            parsed = urlparse(value)
+            if parsed.scheme in ["file", ""] and parsed.netloc == "":
+                getattr(namespace, self.dest).append(parsed.path.rstrip("/"))
+            else:
+                namespace.urls.append(value)
+
+
 def rag_parser(subparsers):
     parser = subparsers.add_parser(
         "rag",
@@ -1257,8 +1347,8 @@ def rag_parser(subparsers):
     )
     parser.add_argument(
         "--image",
-        default=accel_image(CONFIG),
-        help="OCI container image to run with the specified AI model",
+        default=rag_image(CONFIG),
+        help="Image to use for generating RAG data",
         action=OverrideDefaultAction,
         completer=local_images,
     )
@@ -1286,11 +1376,14 @@ If GPU device on host is accessible to via group access, this option leaks the u
         help="Enable SELinux container separation",
     )
     parser.add_argument(
-        "PATH",
-        nargs="*",
-        help="""\
-Files/Directory containing PDF, DOCX, PPTX, XLSX, HTML, AsciiDoc & Markdown
-formatted files to be processed""",
+        "PATHS",
+        nargs="+",
+        help=dedent(
+            """
+        Files/URLs/Directory containing PDF, DOCX, PPTX, XLSX, HTML, AsciiDoc & Markdown
+        formatted files to be processed"""
+        ),
+        action=AddPathOrUrl,
     )
     parser.add_argument(
         "DESTINATION", help="Path or OCI Image name to contain processed rag data", completer=suppressCompleter
@@ -1307,7 +1400,8 @@ formatted files to be processed""",
 
 def rag_cli(args):
     rag = Rag(args.DESTINATION)
-    rag.generate(args)
+    args.inputdir = INPUT_DIR
+    rag.generate(args, assemble_command(args))
 
 
 def rm_parser(subparsers):
@@ -1320,9 +1414,7 @@ def rm_parser(subparsers):
 
 def _rm_model(models, args):
     for model in models:
-        resolved_model = shortnames.resolve(model)
-        if resolved_model:
-            model = resolved_model
+        model = shortnames.resolve(model)
 
         try:
             m = New(model, args)
@@ -1377,7 +1469,8 @@ def perplexity_parser(subparsers):
 
 def perplexity_cli(args):
     model = New(args.MODEL, args)
-    model.perplexity(args)
+    model.ensure_model_exists(args)
+    model.perplexity(args, assemble_command(args))
 
 
 def inspect_parser(subparsers):
@@ -1402,8 +1495,13 @@ def inspect_cli(args):
     model.inspect(args.all, args.get == "all", args.get, args.json, args.dryrun)
 
 
-def main():
+def main() -> None:
     def eprint(e, exit_code):
+        try:
+            if args.debug:
+                logger.exception(e)
+        except Exception:
+            pass
         perror("Error: " + str(e).strip("'\""))
         sys.exit(exit_code)
 
@@ -1421,7 +1519,7 @@ def main():
         if not args.subcommand:
             parser.print_usage()
             perror("ramalama: requires a subcommand")
-            return 0
+            return
 
         args.func(args)
     except urllib.error.HTTPError as e:
@@ -1432,6 +1530,8 @@ def main():
         eprint(e, errno.EINVAL)
     except NotImplementedError as e:
         eprint(e, errno.ENOSYS)
+    except subprocess.TimeoutExpired as e:
+        eprint(e, errno.ETIMEDOUT)
     except subprocess.CalledProcessError as e:
         eprint(e, e.returncode)
     except EndianMismatchError:
