@@ -3,12 +3,23 @@ from unittest.mock import patch
 
 import pytest
 
-from ramalama.config import DEFAULT_PORT, default_config, get_default_engine, get_default_store, load_env_config
+from ramalama.config import (
+    DEFAULT_PORT,
+    BaseConfig,
+    RamalamaImages,
+    default_config,
+    get_default_engine,
+    get_default_store,
+    load_env_config,
+)
+from ramalama.log_levels import LogLevel
 
 
 def test_correct_config_defaults(monkeypatch):
     monkeypatch.delenv("RAMALAMA_IMAGE", raising=False)
-    cfg = default_config()
+    with patch("ramalama.config.load_file_config", return_value={}):
+        with patch("ramalama.config.load_env_config", return_value={}):
+            cfg = default_config()
 
     assert cfg.carimage == "registry.access.redhat.com/ubi10-micro:latest"
     assert cfg.container in [True, False]  # depends on env/system
@@ -18,23 +29,26 @@ def test_correct_config_defaults(monkeypatch):
     assert cfg.env == []
     assert cfg.host == "0.0.0.0"
     assert cfg.image == cfg.default_image
-    assert isinstance(cfg.images, dict)
+    assert isinstance(cfg.images, RamalamaImages)
     assert cfg.api == "none"
     assert cfg.keep_groups is False
     assert cfg.ngl == -1
     assert cfg.threads == -1
     assert cfg.port == str(DEFAULT_PORT)
-    assert cfg.pull == "newer"
+    assert cfg.pull in ["newer", "always"]  # depends on engine
     assert cfg.runtime == "llama.cpp"
     assert cfg.store == get_default_store()
     assert cfg.temp == "0.8"
     assert cfg.transport == "ollama"
     assert cfg.ocr is False
+    assert cfg.verify is True
 
 
 def test_config_defaults_not_set(monkeypatch):
     monkeypatch.delenv("RAMALAMA_IMAGE", raising=False)
-    cfg = default_config()
+    with patch("ramalama.config.load_file_config", return_value={}):
+        with patch("ramalama.config.load_env_config", return_value={}):
+            cfg = default_config()
 
     assert cfg.is_set("carimage") is False
     assert cfg.is_set("container") is False  # depends on env/system
@@ -55,6 +69,17 @@ def test_config_defaults_not_set(monkeypatch):
     assert cfg.is_set("temp") is False
     assert cfg.is_set("transport") is False
     assert cfg.is_set("ocr") is False
+    assert cfg.is_set("verify") is False
+
+
+def test_base_config_normalizes_pull_for_docker():
+    config = BaseConfig(engine="docker", pull="newer")
+    assert config.pull == "always"
+
+
+def test_base_config_preserves_pull_for_non_docker():
+    config = BaseConfig(engine="podman", pull="newer")
+    assert config.pull == "newer"
 
 
 def test_file_config_overrides_defaults():
@@ -62,6 +87,7 @@ def test_file_config_overrides_defaults():
         "image": "custom/image:latest",
         "threads": 8,
         "container": False,
+        "verify": False,
     }
 
     with patch("ramalama.config.load_file_config", return_value=mock_file_config):
@@ -70,10 +96,12 @@ def test_file_config_overrides_defaults():
             assert cfg.image == "custom/image:latest"
             assert cfg.threads == 8
             assert cfg.container is False
+            assert cfg.verify is False
 
             assert cfg.is_set("image") is True
             assert cfg.is_set("threads") is True
             assert cfg.is_set("container") is True
+            assert cfg.is_set("verify") is True
 
 
 def test_env_overrides_file_and_default():
@@ -138,8 +166,32 @@ def test_cfg_container_not_set():
             assert cfg.container is (cfg.engine is not None)
 
 
-class TestGetDefaultEngine:
+def test_base_config_accepts_none_log_level():
+    cfg = BaseConfig(engine=None)
+    assert cfg.log_level is None
 
+
+def test_base_config_coerces_log_level_strings():
+    cfg = BaseConfig(engine=None, log_level="info")
+    assert cfg.log_level == LogLevel.INFO
+
+
+def test_load_env_config_coerces_log_level():
+    cfg = load_env_config({"RAMALAMA_LOG_LEVEL": "debug"})
+    assert cfg["log_level"] == LogLevel.DEBUG
+
+
+@pytest.mark.xfail(raises=ValueError)
+def test_load_env_config_invalid_log_level_raises():
+    load_env_config({"RAMALAMA_LOG_LEVEL": "notalevel"})
+
+
+@pytest.mark.xfail(raises=ValueError)
+def test_load_env_config_invalid_log_level_case_raises():
+    load_env_config({"RAMALAMA_LOG_LEVEL": "InVaLiD"})
+
+
+class TestGetDefaultEngine:
     def test_get_default_engine_with_toolboxenv(self):
         with patch("os.getenv", return_value=None):
             with patch("os.path.exists", side_effect=lambda x: x == "/run/.toolboxenv"):
@@ -165,13 +217,27 @@ class TestGetDefaultEngine:
     def test_get_default_engine_with_docker_available_osx(self):
         with patch("ramalama.config.available", side_effect=lambda x: x == "docker"):
             with patch("sys.platform", "darwin"):
-                assert get_default_engine() is None
+                assert get_default_engine() == "docker"
 
     def test_get_default_engine_with_docker_available_linux(self):
         with patch("ramalama.config.available", side_effect=lambda x: x == "docker"):
             with patch("sys.platform", "linux"):
                 assert get_default_engine() == "docker"
 
+    def test_default_engine_falls_back_to_docker_when_podman_machine_missing(self):
+        with (
+            patch("ramalama.config.available", side_effect=lambda binary: binary in {"podman", "docker"}),
+            patch("ramalama.config.apple_vm", return_value=False),
+            patch("ramalama.config.load_file_config", return_value={}),
+            patch("ramalama.config.load_env_config", return_value={}),
+            patch("ramalama.config.os.path.exists", return_value=False),
+            patch("ramalama.config.sys.platform", "darwin"),
+        ):
+            cfg = default_config()
+
+        assert cfg.engine == "docker"
+        assert cfg.is_set("engine") is False
+
 
 class TestLoadEnvConfig:
     """Test the load_env_config function for arbitrary environment variable loading."""
@@ -183,6 +249,7 @@ class TestLoadEnvConfig:
             "RAMALAMA_THREADS": "8",
             "RAMALAMA_CONTAINER": "true",
             "RAMALAMA_HOST": "127.0.0.1",
+            "RAMALAMA_VERIFY": "false",
         }
 
         result = load_env_config(env)
@@ -192,6 +259,7 @@ class TestLoadEnvConfig:
             "threads": 8,
             "container": True,
             "host": "127.0.0.1",
+            "verify": False,
         }
         assert result == expected
 
@@ -483,6 +551,7 @@ class TestConfigIntegration:
             assert cfg.threads == 8
             assert cfg.user.no_missing_gpu_prompt is True
             assert cfg.images["CUDA_VISIBLE_DEVICES"] == "env/cuda:latest"
+            assert cfg.images["HIP_VISIBLE_DEVICES"] == "quay.io/ramalama/rocm"
 
     def test_config_multiple_env_layers(self):
         """Test that multiple environment variable layers work correctly."""
@@ -586,11 +655,13 @@ class TestConfigIntegration:
             assert cfg.user.no_missing_gpu_prompt is True
 
             # Deep merged images
-            expected_images = {
-                "CUDA_VISIBLE_DEVICES": "custom/cuda:latest",  # from env
-                "INTEL_VISIBLE_DEVICES": "custom/intel:latest",  # from env
-                "HIP_VISIBLE_DEVICES": "quay.io/ramalama/rocm:latest",  # from file config
-            }
+            expected_images = RamalamaImages(
+                **{
+                    "CUDA_VISIBLE_DEVICES": "custom/cuda:latest",  # from env
+                    "INTEL_VISIBLE_DEVICES": "custom/intel:latest",  # from env
+                    "HIP_VISIBLE_DEVICES": "quay.io/ramalama/rocm:latest",  # from file config
+                }
+            )
             assert cfg.images == expected_images
 
     def test_config_is_set_behavior(self):
